module Api
  class HandlerBase

    #
    # Properties
    #

    attr_reader :requests_count
    attr_reader :handle_duration_total
    attr_reader :handle_duration_max
    attr_reader :handle_duration_min

    #
    # Methods
    #

    # default constructor
    def initialize(options = nil)
      @stats_locker = Mutex.new

      reset_handler_stats

      @logger = AppInfo.instance.logger
    end

    public
    def before_handle
    end

    public
    def handle_request(session, request)
    end

    public
    def post_handle(handle_start_at)
      return if handle_start_at.nil?

      handle_duration = ((Time.now - handle_start_at).to_f * 1000).to_i

      @stats_locker.synchronize do
        @requests_count        += 1
        @handle_duration_total += handle_duration

        @handle_duration_min = [@handle_duration_min, handle_duration].min
        @handle_duration_max = [@handle_duration_max, handle_duration].max
      end
    end

    public
    def reset_handler_stats
      @stats_locker.synchronize do
        @requests_count        = 0
        @handle_duration_total = 0

        @handle_duration_max = 0
        @handle_duration_min = 2_147_483_647
      end
    end

    protected
    def player_is_nil?(session, request_id)
      result = session.player.nil?

      if result
        session.send_response_error(request_id)

        AppInfo.instance.on_server_error("player is nil in handler #{self.class}")
      end

      result
    end

    protected
    def send_response_ok_unless_error(session, request_id, error = nil)
      if error.nil?
        response = Api::ResponseOk.new(request_id)
      else
        response             = Api::ResponseError.new(request_id)
        response.description = error
      end

      session.send_response(response)
    end

  end
end
