#todo:review
class ModelBase

  include AMFSerializable

  #
  # admin
  #

  #
  # properties
  #

  public
  def self.admin_group
    self.name.underscore.split('_').first
  end

  public
  def self.configure_admin_list(context, field_name)
    model_class = get_class_by_name(context.abstract_model.model_name)

    is_mongoid_field          = false
    is_mongoid_relation_field = false

    if model_class.include?(Mongoid::Document)
      if model_class.fields.include?(field_name.to_s)
        is_mongoid_field = true
      end

      if model_class.relations.keys.include?(field_name.to_s)
        is_mongoid_relation_field = true
      end
    end

    if is_mongoid_field && model_class.fields[field_name.to_s].type == Mongoid::Boolean
      context.configure field_name
    elsif is_mongoid_relation_field
      context.configure field_name
    else
      case field_name
        when :_type, /\Auploader_/
          context.configure field_name do
            visible false
          end
        when /_at\z/
          context.configure field_name do
            pretty_value do
              Time.at(value).to_s(:short) unless value.nil?
            end
            read_only true
          end
        when /\Aurl_/
          context.configure field_name do
            pretty_value do
              unless value.nil?
                UtilsAdmin.tag_img(bindings[:view], value, 25, 25)
              end
            end
          end
        else
          context.configure field_name do
            pretty_value { value }
          end
      end
    end
  end

  public
  def self.configure_admin_edit(context, field_name)
    case field_name
      when :_type, :created_at, :updated_at
        context.configure field_name do
          visible false
        end
      else
        context.configure field_name
    end
  end

  public
  def self.configure_admin_show(context, field_name)
    case field_name

      when :_type
        context.configure field_name do
          visible false
        end

      when /_at\z/
        context.configure field_name do
          pretty_value do
            Time.at(value).to_s(:short)
          end
        end

      when /\Aurl_/
        context.configure field_name do
          pretty_value do
            UtilsAdmin.tag_img(bindings[:view], value, 100, 100)
          end
        end

      else
        context.configure field_name
    end
  end

  public
  def self.sort_by_for_admin_list(section)
    section.sort_by :_id
  end

  public
  def self.filters_for_admin_list(section)
    section.filters []
  end

  #
  # scopes
  #

  public
  def self.init_scopes

  end

  protected
  def self.add_scope(scope_name, criteria)
    scope scope_name.to_sym, -> { where(criteria) }
  end

  #
  # Properties
  #

  protected
  def class_info
    @class_info = nil
  end

  #
  # Methods
  #

  # default constructor
  def initialize(options = nil)
    super
  end

  public
  def create_info
    return nil if class_info.nil?

    result = class_info.new

    class_info.attributes.each do |attribute_name|
      next unless self.respond_to? attribute_name

      attribute_value = get_attribute_value(attribute_name)

      result.send("#{attribute_name}=", attribute_value)
    end

    result
  end

  public
  def create_info_with_check
    return nil if class_info.nil?

    result = class_info.new

    class_info.attributes.each do |attribute_name|
      if (self.respond_to?(attribute_name) && self.has_attribute?(attribute_name)) || attribute_name.to_s == 'id'

        attribute_value = get_attribute_value(attribute_name)

        result.send("#{attribute_name}=", attribute_value)
      end
    end

    result
  end

  protected
  def get_attribute_value(attribute_name)
    result = self.send(attribute_name)

    if result.is_a? (Array)
      result_array = []

      result.each do |attribute_element|
        if attribute_element.kind_of? (ModelBase)
          result_array << attribute_element.create_info
        else
          result_array << attribute_element
        end
      end

      result = result_array

    elsif result.kind_of? (ModelBase)
      result = result.create_info
    end

    result
  end

  public
  def clone_model
    self.class.new(self.as_json)
  end

end