class BundleBase < ModelBase

  include Mongoid::Document
  prepend MongoidIdAlias

  include Mongoid::Timestamps

  include UploaderHelperLocal

  #
  # admin
  #

  public
  def self.configure_admin_list(section, field_name)
    case field_name
      when :_id, :url, :created_at
        section.configure field_name do
          visible false
        end
      else
        super(section, field_name)
    end
  end

  public
  def self.configure_admin_edit(section, field_name)
    case field_name
      when :build_type
        section.configure field_name, :enum do
          enum(EBuildType.values_list)
        end
      when :version
        section.configure field_name do
          read_only true
        end
      when :url
        section.configure field_name do
          read_only true
          help 'Required. Please upload file'
        end

      else
        super(section, field_name)
    end
  end

  #
  # callbacks
  #

  after_validation :update_version

  #
  # db
  #

  store_in collection: 'bundles'

  #
  # fields
  #

  field :build_type, type: String
  field :name, type: String
  field :url, type: String

  field :crc, type: Integer, default: 0
  field :version, type: Integer, default: 0

  #
  # indexes
  #

  index({build_type: 1, name: 1}, {unique: true})

  #
  # Properties
  #

  private
  def class_info
    BundleInfoBase
  end

  #
  # Methods
  #

  # default constructor
  public
  def initialize(options = nil)
    super
  end

  #
  # validates
  #

  validates :name, presence: true, allow_blank: false, allow_nil: false
  validates :url, presence: true, allow_blank: false, allow_nil: false
  validates :build_type, presence: true, allow_blank: false, allow_nil: false

  #
  # callbacks
  #

  public
  def update_version
    return unless errors.empty?

    return unless self.changed?

    self.version += 1
  end

  #
  # uploaders
  #

  public
  mount_uploader :uploader_bundle, FileUploader

  protected
  def get_path_for_uploader_relative(field_name)
    "#{Constants::DIR_ASSETS_DYNAMIC}/bundles/#{self.build_type}/#{self.name}"
  end

  protected
  def url_uploaders_map
    {
        :url => :uploader_bundle
    }
  end

  #
  # scopes
  #

  public
  def self.init_scopes
    bundles = self.only(:build_type, :name).to_a

    names_list = []

    bundles.each do |bundle|
      name = bundle.name.underscore.split('_').first

      name = name.split('/').first

      names_list << name
    end

    names_list.uniq!

    names_list.each do |name|
      scope_name = "#{name}"

      criteria = {name: /^#{name}.*/i}

      add_scope(scope_name, criteria)
    end

    EBuildType.values_list.each do |build_type|
      criteria = {build_type: build_type, }
      add_scope(build_type, criteria)
    end
  end

end