class ErrorServerBase < ModelBase

  include Mongoid::Document
  prepend MongoidIdAlias

  #
  # Static Methods
  #

  public
  def self.configure_admin_list(section, field_name)

    case field_name
      when :_id, :md5, :stack_trace
        section.configure field_name do
          visible false
        end

      when :throws_count, :players_count
        section.configure field_name do
          column_width 60
        end

      else
        super(section, field_name)
    end

  end

  public
  def self.configure_admin_show(section, field_name)

    case field_name
      when :stack_trace
        section.configure field_name do
          pretty_value do
            value.join('<br>').html_safe
          end
        end

      else
        super(section, field_name)
    end
  end

  public
  def self.sort_by_for_admin_list(section)

    section.sort_by :last_error_at
    section.configure :last_error_at do
      sort_reverse true
    end

  end

  #
  # db
  #

  store_in collection: 'system_error_servers'

  #
  # fields
  #

  #md5 from message + request_class
  field :md5

  field :last_error_at, type: Integer
  field :first_error_at, type: Integer
  field :message, type: String
  field :request_class, type: String
  field :players_ids_map, type: Hash, default: {}
  field :throws_count, type: Integer, default: 0
  field :players_count, type: Integer, default: 0
  field :stack_trace, type: Array

  #
  # add indexes
  #

  index(md5: 1)

  #
  # Methods
  #

  # default constructor
  def initialize(options = nil)
    super(options)
  end

end