class ManagerRemoteSocketBase < ManagerBase

  include ManagerRemoteSocketApi
  include ManagerRemoteSocketDisconnect
  include ManagerRemoteSocketHandle
  include ManagerRemoteSocketSessions
  include ManagerRemoteSocketStatistic
  include NetworkInterfaceHelper

  #
  # Properties
  #

  attr_reader :ip
  attr_reader :port
  attr_reader :port_secure

  attr_reader :sessions

  protected
  def default_port
    assert(false, 'please override')
  end

  protected
  def server_type
    'unknown'
  end

  public
  def is_ready_for_shutdown?
    result = false

    @sessions_locker.synchronize do
      result = @sessions.empty?
    end

    result
  end

  #
  # Events
  #

  public
  def on_server_shutdown
    super

    disconnect_all_with_message(EStringTypeBase::EST_ERROR_SERVER_SHUTDOWN)

    nil
  end

  public
  def on_server_maintenance_on
    self.disconnect_all_with_message(EStringTypeBase::EST_ERROR_SERVER_ON_MAINTENANCE)

    nil
  end

  #
  # Methods
  #

  # default constructor
  public
  def initialize
    super

    @port        = 0
    @port_secure = 0

    init_api
    init_handle
    init_sessions
  end

  public
  def post_init
    super

    register_classes
    register_handlers

    nil
  end

  public
  def start_tasks
    Constants::THREADS_HANDLERS_COUNT.times do |n|
      create_handler(n)
    end

    nil
  end

  def start!(callback = nil)
    begin
      descriptor_size = EventMachine.set_descriptor_table_size(60000)

      @logger.info "descriptors size is #{descriptor_size}"

    rescue Exception => e
      AppInfo.instance.on_server_error(e.to_s, e)
    end
  end

  protected
  def start_server_on_port(port, client_class, continue_after_error = true, callback = nil)
    thread = Thread.new do

      client_class.manager_remote = self if client_class.respond_to?('manager_remote=')

      begin

        EventMachine.epoll

        EventMachine.run do

          ip = '0.0.0.0'

          EventMachine.start_server(ip, port, client_class)

          @logger.info "server #{self.server_type}\t\tstarted on #{ip}:#{port}"

          callback.call unless callback.nil?
        end
      rescue Exception => e
        message = "error on server #{ip}:#{port} thread: #{e}"

        AppInfo.instance.on_server_error(message, e)

        sleep 10

        retry if continue_after_error

      end
    end

    thread.name = "thread_remote_server_#{ip}:#{port}"

    nil
  end

end