class SocketClientBase < EM::Connection

  #
  # static fields
  #

  public
  def self.manager_remote=(value)
    @manager_remote = value
  end

  public
  def self.manager_remote
    @manager_remote
  end

  #
  # Properties
  #
  attr_reader :time_last_request
  attr_reader :ip

  #use for disconnect_inactive_players and on_application_exit
  attr_accessor :should_disconnect_session

  attr_accessor :player

  attr_accessor :callback_on_disconnect

  public
  def is_online?
    @is_online
  end

  #
  # Events
  #

  protected
  def on_get_requests_error(e)
    AppInfo.instance.on_server_error("Error on get_requests #{e}", e)
  end

  protected
  def on_connection_closed
    @is_online = false

    @logger.info "connection closed. ip: #{@ip}"

    @manager_remote.on_client_disconnected(self) if @should_disconnect_session

    unless @callback_on_disconnect.nil?
      @callback_on_disconnect.call
      @callback_on_disconnect = nil
    end

    nil
  end

  #
  # Methods
  #

  # default constructor
  public
  def post_init
    @logger = AppInfo.instance.logger

    @requests_locker = Mutex.new

    @is_online = true

    @time_last_request = Time.now.to_i

    @should_disconnect_session = true

    @manager_remote = self.class.manager_remote
    @manager_remote.on_client_connected(self)

    @data_buffer = ''

    @player                 = nil
    @callback_on_disconnect = nil

    init_ip

    nil
  end

  protected
  def init_ip
    init_ip_internal(get_peername)

    nil
  end

  protected
  def init_ip_internal(peername)
    @ip = Socket.unpack_sockaddr_in(peername).last rescue '0.0.0.0'

    @logger.info "connection opened. ip: #{@ip}"

    nil
  end

  public
  def receive_data(data)
    @time_last_request = Time.now.to_i

    @requests_locker.synchronize do
      @data_buffer += data
    end

    @manager_remote.handle_data_from(self)

    nil
  end

  public
  def get_requests
    result = []

    return result unless @is_online

    @requests_locker.synchronize do
      begin

        unless @data_buffer.empty?
          deserialize_data = AMF::Root.deserialize(@data_buffer)
          result           = deserialize_data[:objects]
          @data_buffer     = deserialize_data[:incomplete_objects] || ''
        end

      rescue Exception => e
        on_get_requests_error(e)
      end #begin/rescue
    end #synchronize

    result
  end


  public
  def send_response(response_obj)
    return unless @is_online

    report_about_sent_request(response_obj)

    response = AMF::Root.serialize(response_obj)

    send_data(response)

    nil #for prevent return other value
  end

  if Rails.env.development?
    protected
    def report_about_sent_request(value)
      value.report_log("sent\t\t") if value.respond_to?(:report_log)

      nil
    end
  else
    protected
    def report_about_sent_request(value)
      #do nothing
    end
  end


  public
  def send_raw_data(raw_data)
    send_data(raw_data)
  end

  public
  def send_response_ok(request_id)
    response = Api::ResponseOk.new(request_id)

    send_response(response)
  end

  public
  def send_response_error(request_id, description = nil)
    response             = Api::ResponseError.new(request_id)
    response.description = description

    send_response(response)
  end

  public
  def unbind
    on_connection_closed

    nil
  end

end