class SocketClientWebSockets < SocketClientBase

  #
  # events
  #


  # Access properties on the EM::WebSocket::Handshake object, e.g.
  # path, query_string, origin, headers
  protected
  def on_connection_open(handshake)
  end

  protected
  def on_error(e)
    AppInfo.instance.on_server_error(e.to_s, e)
  end

  #
  # properties
  #

  public
  def websocket=(value)
    @websocket = value

    init_ip_internal(@websocket.get_peername)

    @websocket.onopen do |handshake|
      begin
        on_connection_open(handshake)
      rescue Exception => e
        on_error(e)
      end
    end

    @websocket.onclose do
      begin
        on_connection_closed
      rescue Exception => e
        on_error(e)
      end
    end

    @websocket.onmessage do |data|
      begin
        receive_data(data)
      rescue Exception => e
        on_error(e)
      end
    end

    @websocket.onbinary do |data|
      begin
        receive_data(data)
      rescue Exception => e
        on_error(e)
      end
    end

    nil
  end

  #
  # methods
  #

  # default constructor
  public
  def post_init
    super
  end

  protected
  def init_ip
    #do nothing
  end

  public
  def send_response(response_obj)
    return unless @is_online

    report_about_sent_request(response_obj)

    response = AMF::Root.serialize(response_obj)

    try_send_binary(response)

    nil #for prevent return other value
  end

  public
  def send_raw_data(response)
    try_send_binary(response)

    nil
  end

  public
  def close_connection_after_writing
    @websocket.close_connection_after_writing
  end

  private
  def try_send_binary(response)
    return unless @is_online

    begin
      @websocket.send_binary(response)
    rescue
      #do nothing
    end

    nil
  end

end