class LocalizationBase < ModelBase

  SUPPORTED_LANGUAGES = ELanguageType.values_list.map(&:to_sym)

  include Mongoid::Document
  prepend MongoidIdAlias

  #
  # admin
  #

  public
  def self.configure_admin_list(section, field_name)
    case field_name
      when :_id
        section.configure field_name do
          visible false
        end
      else
        super(section, field_name)
    end
  end

  #
  # db
  #

  store_in collection: 'localizations'

  #
  # callbacks
  #
  before_save :callback_before_save

  #
  # fields
  #

  field :key, type: String

  SUPPORTED_LANGUAGES.each do |language|
    field language.to_sym, type: String, default: ''
  end

  #
  # add indexes
  #

  index({key: 1}, {unique: true})

  #
  # Methods
  #

  # default constructor
  def initialize(options = nil)
    super
  end

  public
  def callback_before_save
    self.key = self.key.downcase unless self.key.nil?
  end


  #
  # validates
  #

  validates :key, presence: true, uniqueness: true

end