require 'rails_admin/config/actions'
require 'rails_admin/config/actions/base'

module RailsAdmin
  module Config
    module Actions
      class Localization < Base
        RailsAdmin::Config::Actions.register(self)

        register_instance_option :visible? do
          authorized? && (bindings[:abstract_model] == LocalizationBase)
        end

        register_instance_option :collection do
          true
        end

        register_instance_option :link_icon do
          'icon-none'
        end

        register_instance_option :pjax? do
          false
        end

        register_instance_option :http_methods do
          [:get, :post]
        end

        register_instance_option :controller do

          proc do

            model_name = @abstract_model.model_name

            unless model_name == 'LocalizationBase'
              flash[:error] = 'Localization actions work only with Localization model'
              redirect_to index_path
              next
            end

            flash.clear

            next unless request.post?

            params = request.filtered_parameters.symbolize_keys

            if params[:export] # EXPORT

              begin

                manager_string = AppInfo.instance.manager_string

                file_name = manager_string.export_localizations_to_s3

                flash[:success] = "File #{file_name} was exported"

              rescue Exception => error
                flash[:error] = "Error exporting file. #{error.message}"
              end
            end

            render @action.template_name

          end
        end
      end
    end
  end
end